@@ -2,6 +2,7 @@ use crate::read::{Reader, ReaderEpochs};
 use crate::table::{Table, TableWriteGuard};
 use crate::types::*;
 use slab::Slab;
+use std::cell::UnsafeCell;
 use std::fmt;
 
 /// Operations that update the data held internally. Users mutate the tables by
@@ -11,13 +12,15 @@ use std::fmt;
 /// Users must be careful to guarantee that apply_first and apply_second cause
 /// the tables to end up in the same state. They also must be certain not to use
 /// the return value to mutate the underlying table, since this likely can't be
-/// mimiced in 'apply_second'.
+/// mimiced in 'apply_second', which will lead to divergent tables.
 ///
 /// It is *highly* discouraged to create updates which return mutable references
 /// to the table's internals. E.g:
 ///
-///```compile_fail
+///```
+/// # use active_standby::primitives::UpdateTables;
 /// struct MutableRef {}
+///
 /// impl<'a, T> UpdateTables<'a, Vec<T>, &'a mut T> for MutableRef {
 ///    fn apply_first(&mut self, table: &'a mut Vec<T>) -> &'a mut T {
 ///        &mut table[0]
@@ -54,26 +57,10 @@ use std::fmt;
 /// UpdateTables, this is can cause divergence between the tables since
 /// apply_second isn't aware of this mutation.
 ///
-/// An example implementation taken from collections::vec:
-/// ```compile_fail
-/// struct SwapRemove {
-///     index: usize,
-/// }
-/// impl SwapRemove {
-///     // Not needed, but can be convenient for making both applies the same.
-///     fn apply<'a, T>(&mut self, table: &'a mut Vec<T>) -> T {
-///         table.swap_remove(self.index)
-///     }
-/// }
-/// impl<'a, T> UpdateTables<'a, Vec<T>, T> for SwapRemove {
-///     fn apply_first(&mut self, table: &'a mut Vec<T>) -> T {
-///         self.apply(table)
-///     }
-///     fn apply_second(mut self, table: &mut Vec<T>) {
-///         self.apply(table);
-///     }
-/// }
-/// ```
+/// If the table holds large elements, a user may want to save memory by having
+/// Table<Arc\<T>>. This can be done safely so long as UpdateTables never
+/// mutates the value pointed to (T). UpdateTables may instead only update the Table
+/// by inserting and removing elements.
 pub trait UpdateTables<'a, T, R> {
     fn apply_first(&mut self, table: &'a mut T) -> R;
 
@@ -167,29 +154,37 @@ where
     T: Clone,
 {
     pub fn new(t: T) -> Writer<T> {
-        Self::with_readers_capacity(t, 1024)
-    }
-
-    pub fn with_readers_capacity(t: T, readers_capacity: usize) -> Writer<T> {
-        Writer {
-            table: Arc::new(Table::new(t)),
-            readers: Arc::new(Mutex::new(Slab::with_capacity(readers_capacity))),
-            ops_to_replay: Vec::new(),
-            blocking_readers: std::collections::HashMap::new(),
-        }
+        Self::from_identical(t.clone(), t)
     }
 }
 
 impl<T> Writer<T>
 where
-    T: Default + Clone,
+    T: Default,
 {
     pub fn default() -> Writer<T> {
-        Self::new(T::default())
+        Self::from_identical(T::default(), T::default())
     }
 }
 
 impl<T> Writer<T> {
+    /// Create a Writer object for handling active_standby tables.
+    /// - t1 & t2 are the two tables which will become the active and standby
+    ///   tables. They must be identical; this is left to the user to enforce.
+    pub fn from_identical(t1: T, t2: T) -> Writer<T> {
+        Writer {
+            table: Arc::new(Table::from_identical(t1, t2)),
+            readers: Arc::new(Mutex::new(Slab::with_capacity(1024))),
+            ops_to_replay: Vec::new(),
+            blocking_readers: std::collections::HashMap::new(),
+        }
+    }
+
+    /// Update the max number of readers that we expect to exist.
+    pub fn reserve_readers(&mut self, readers_capacity: usize) {
+        self.readers.lock().unwrap().reserve(readers_capacity)
+    }
+
     // Hangs until the standby table has no readers pointing to it, meaning it
     // is safe for updating.
     fn await_standby_table_free(&mut self) {
@@ -352,22 +347,25 @@ where
     T: Clone,
 {
     pub fn new(t: T) -> SendWriter<T> {
-        SendWriter {
-            writer: Writer::new(t),
-        }
+        Self::from_identical(t.clone(), t)
     }
 }
 
 impl<T> SendWriter<T>
 where
-    T: Default + Clone,
+    T: Default,
 {
     pub fn default() -> SendWriter<T> {
-        Self::new(T::default())
+        Self::from_identical(T::default(), T::default())
     }
 }
 
 impl<T> SendWriter<T> {
+    pub fn from_identical(t1: T, t2: T) -> SendWriter<T> {
+        SendWriter {
+            writer: Writer::from_identical(t1, t2),
+        }
+    }
     pub fn write(&mut self) -> SendWriteGuard<'_, T> {
         SendWriteGuard {
             guard: self.writer.write(),
@@ -413,6 +411,95 @@ impl<'w, T> std::ops::Deref for SendWriteGuard<'w, T> {
     }
 }
 
+#[derive(Debug)]
+pub struct SyncWriter<T> {
+    // Used to lock 'write'. We can't use Mutex<Writer> because then locking
+    // would return MutegGuard<Writer>. Then if we try to crete a WriteGuard,
+    // this would create a struct that refers to itself. Instead we use the C++
+    // style unique_lock, to guard 'writer'.
+    mtx: Mutex<()>,
+
+    // UnsafeCell is used for interior mutability.
+    writer: UnsafeCell<SendWriter<T>>,
+}
+
+impl<T> SyncWriter<T>
+where
+    T: Clone,
+{
+    pub fn new(t: T) -> SyncWriter<T> {
+        Self::from_identical(t.clone(), t)
+    }
+}
+
+impl<T> SyncWriter<T>
+where
+    T: Default,
+{
+    pub fn default() -> SyncWriter<T> {
+        Self::from_identical(T::default(), T::default())
+    }
+}
+
+impl<T> SyncWriter<T> {
+    pub fn from_identical(t1: T, t2: T) -> SyncWriter<T> {
+        SyncWriter {
+            mtx: Mutex::new(()),
+            writer: UnsafeCell::new(SendWriter::from_identical(t1, t2)),
+        }
+    }
+
+    pub fn write(&self) -> SyncWriteGuard<'_, T> {
+        let writer = unsafe { &mut *self.writer.get() };
+        SyncWriteGuard {
+            _mtx_guard: Some(self.mtx.lock().unwrap()),
+            write_guard: Some(UnsafeCell::new(writer.write())),
+        }
+    }
+
+    pub fn new_reader(&self) -> Reader<T> {
+        let _mtx_guard = self.mtx.lock().unwrap();
+        let writer = unsafe { &*self.writer.get() };
+        writer.new_reader()
+    }
+}
+
+/// We enforce Sync by making sure that the WriteGuard only exists whent he
+/// MutexGuard exists.
+unsafe impl<T> Sync for SyncWriter<T> where SyncWriter<T>: Send {}
+
+/// Guard for a SyncWriter, not a WriteGuard that is Sync.
+///
+/// Same as a WriteGuard, but update_tables requires that updates are Send.
+pub struct SyncWriteGuard<'w, T> {
+    _mtx_guard: Option<MutexGuard<'w, ()>>,
+    write_guard: Option<UnsafeCell<SendWriteGuard<'w, T>>>,
+}
+
+impl<'w, T> std::ops::Deref for SyncWriteGuard<'w, T> {
+    type Target = T;
+    fn deref(&self) -> &Self::Target {
+        unsafe { &*&*self.write_guard.as_ref().unwrap().get() }
+    }
+}
+
+impl<'w, T> SyncWriteGuard<'w, T> {
+    pub fn update_tables<'a, R>(
+        &'a mut self,
+        update: impl UpdateTables<'a, T, R> + 'static + Sized + Send,
+    ) -> R {
+        unsafe { &mut *self.write_guard.as_ref().unwrap().get() }.update_tables(update)
+    }
+}
+
+impl<'w, T> Drop for SyncWriteGuard<'w, T> {
+    fn drop(&mut self) {
+        // Make sure to destroy the write guard before the mutex.
+        self.write_guard = None;
+        self._mtx_guard = None;
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
